#include "VideoView.h"
#include "../../../../pokktsdk/PokktCocosManager.h"
#include "../Storage.h"

using namespace cocos2d;
using namespace ui;

PokktCocosConfig *VideoView::pokktCocosConfig = new PokktCocosConfig();

bool VideoView::init()
{
    if (!BaseView::init())
        return false;
    
    _coinsEarned = 0;
    
    logo->setPosition(logo->getPosition().x, logo->getPosition().y + 100);
    
    int yPos = logo->getPosition().y - logo->getBoundingBox().size.height / 2;
    
    auto lblPointEarned = BaseView::createLabel("Points Earned: ", 35, Color3B::BLACK, Vec2(visibleSize.width / 2 - 30, yPos));
    lblPointEarned->setTextHorizontalAlignment(TextHAlignment::RIGHT);
    this->addChild(lblPointEarned);
    
    _txtPointsEarned = BaseView::createLabel("0", 35, Color3B::BLACK,
                                             Vec2(lblPointEarned->getBoundingBox().size.width / 2 + lblPointEarned->getPosition().x + 20, yPos));
    this->addChild(_txtPointsEarned);
    
    yPos -= 100;
    
    _txtCacheMessage = createLabel("Getting Video Campaign, please wait...", 25, Color3B::WHITE, Vec2(visibleSize.width / 2, yPos));
    _txtCacheMessage->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txtCacheMessage);
    
    yPos -= 120;
    
    _btnStartCaching = createButton("Start Caching", CC_CALLBACK_1(VideoView::handleStartCaching, this));
    _btnStartCaching->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnStartCaching->setVisible(!Storage::getAutoCaching());
    this->addChild(_btnStartCaching);
    
    yPos -= 100;
    
    _btnPlayVideo = createButton("Play Video to Earn Coins", CC_CALLBACK_1(VideoView::handlePlayVideo, this));
    _btnPlayVideo->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnPlayVideo->setVisible(false);
    this->addChild(_btnPlayVideo);
    
    yPos -= 100;
    
    _btnPlayVideoNonIncent = createButton("Play Non Incentive Video", CC_CALLBACK_1(VideoView::handlePlayVideoNonIncent, this), "button_purple.png");
    _btnPlayVideoNonIncent->setPosition(Vec2(visibleSize.width / 2, yPos));
    _btnPlayVideoNonIncent->setVisible(false);
    this->addChild(_btnPlayVideoNonIncent);
    
    yPos -= 100;
    
    pokktCocosConfig->setSecurityKey("iJ02lJss0M");
    pokktCocosConfig->setApplicationId("a2717a45b835b5e9f50284a38d62a74e");
    pokktCocosConfig->setIntegrationType(PokktCocosConfig::INTEGRATION_TYPE_ALL);
    
    // set auto caching
    pokktCocosConfig->setAutoCacheVideo(Storage::getAutoCaching());
    
    // set params to init pokkt
    PokktCocosManager::initPokkt(pokktCocosConfig);
    
    // pokkt init for video ads
    pokktCocosConfig->setThirdPartyUserId("123456");
    pokktCocosConfig->setSkipEnabled(true);
    pokktCocosConfig->setDefaultSkipTime(10);
    
    if (PokktCocosManager::isVideoAvailable())
    {
        double vc = PokktCocosManager::getVideoVc();
        std::stringstream vcStr;
        vcStr << vc;
        handleDownloadCompleted(vcStr.str());
    }
    
    // listen for pokkt events
    PokktCocosManager::setListener(PokktCocosManager::VideoClosedEvent, pokkt_event_selector(VideoView::handleVideoClosed), this);
    PokktCocosManager::setListener(PokktCocosManager::VideoDisplayedEvent, pokkt_event_selector(VideoView::handleVideoDisplayed), this);
    PokktCocosManager::setListener(PokktCocosManager::VideoSkippedEvent, pokkt_event_selector(VideoView::handleVideoSkipped), this);
    PokktCocosManager::setListener(PokktCocosManager::VideoCompletedEvent, pokkt_event_selector(VideoView::handleVideoCompleted), this);
    PokktCocosManager::setListener(PokktCocosManager::VideoGratifiedEvent, pokkt_event_selector(VideoView::handleVideoGratified), this);
    
    PokktCocosManager::setListener(PokktCocosManager::DownloadCompletedEvent, pokkt_event_selector(VideoView::handleDownloadCompleted), this);
    PokktCocosManager::setListener(PokktCocosManager::DownloadFailedEvent, pokkt_event_selector(VideoView::handleDownloadFailed), this);
    
    return true;
}

VideoView::~VideoView()
{
    PokktCocosManager::removeListener(PokktCocosManager::VideoClosedEvent);
    PokktCocosManager::removeListener(PokktCocosManager::VideoDisplayedEvent);
    PokktCocosManager::removeListener(PokktCocosManager::VideoSkippedEvent);
    PokktCocosManager::removeListener(PokktCocosManager::VideoCompletedEvent);
    PokktCocosManager::removeListener(PokktCocosManager::VideoGratifiedEvent);
    
    PokktCocosManager::removeListener(PokktCocosManager::DownloadCompletedEvent);
    PokktCocosManager::removeListener(PokktCocosManager::DownloadFailedEvent);
}

void VideoView::handleBackPress(cocos2d::Ref *pSender)
{
    changeView(MainView);
}

void VideoView::handleStartCaching(cocos2d::Ref *pSender)
{
    PokktCocosManager::cacheVideoCampaign();
}

void VideoView::handlePlayVideo(cocos2d::Ref *pSender)
{
    pokktCocosConfig->setScreenName("sample_app");
    PokktCocosManager::getVideo(pokktCocosConfig);
}

void VideoView::handlePlayVideoNonIncent(cocos2d::Ref *pSender)
{
    pokktCocosConfig->setScreenName("sample_app");
    PokktCocosManager::getVideoNonIncent(pokktCocosConfig);
}

void VideoView::handleVideoClosed(std::string message)
{
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message.c_str());
    // if auto caching flow -- handleDownloadCompleted should fire again
    // if not auto cache flow -- user should click Start Caching button again
    _btnPlayVideo->setVisible(false);
    _btnPlayVideoNonIncent->setVisible(false);
}

void VideoView::handleVideoDisplayed(std::string message)
{
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message.c_str());
}

void VideoView::handleVideoSkipped(std::string message)
{
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message.c_str());
}

void VideoView::handleVideoCompleted(std::string message)
{
    CCLOG("[POKKT-CPP] %s", message.c_str());
    _txtCacheMessage->setString(message.c_str());
}

void VideoView::handleVideoGratified(std::string coins)
{
    CCLOG("[POKKT-CPP] Coins earned from video: %s", coins.c_str());
    
    int coinsInt = 0;
    std::stringstream(coins) >> coinsInt;
    _coinsEarned += coinsInt;
    
    char buf[100];
    sprintf(buf, "%d", _coinsEarned);
    _txtPointsEarned->setString(buf);
}

void VideoView::handleDownloadCompleted(std::string coinsToEarn)
{
    double vc = PokktCocosManager::getVideoVc();
    
    // enable buttons
    CCLOG("[POKKT-CPP] enabling buttons! Video vc is: %f", vc);
    
    _txtCacheMessage->setString("Video downloaded!");
    
    //_btnPlayVideo->setTitleText("Play Video to Earn " + coinsToEarn);
    _btnPlayVideo->setVisible(true);
    _btnPlayVideoNonIncent->setVisible(true);
    //_btnPlayVideoWithIntent->setVisible(true);
}

void VideoView::handleDownloadFailed(std::string message)
{
    CCLOG("[POKKT-CPP] Failed to download video!");
    _txtCacheMessage->setString("Failed to download video!");
}
