#import "PokktCocosManager.h"
#import <Foundation/Foundation.h>
#import "PokktManager.h"
#import "VideoResponse.h"
#import "IOSExtension.h"

// use this to notify the cocos2dx
static void NotifyCocos(const char* operation, const char*  param)
{
    PokktCocosManager::handleNativeMessage(operation, param);
}

PokktConfig *pokktConfig;

@interface IOSExtension:NSObject<PokktDelegate>
@end

@implementation IOSExtension

#pragma mark - Pokkt Delegates

- (void)didFinishedVideoDownload:(BOOL)isFinished
{
    float vc = [PokktManager getVideoVc];
    NotifyCocos(PokktCocosManager::DownloadCompletedEvent.c_str(), [[NSString stringWithFormat:@"%@", [[NSNumber numberWithFloat:vc] stringValue]] UTF8String]);
}

- (void)didFailedVideoDownload:(NSString *)error
{
    NotifyCocos(PokktCocosManager::DownloadFailedEvent.c_str(), [[NSString stringWithFormat:@"%@", error] UTF8String]);
}

- (void)onVideoCompleated:(BOOL)isCompleated
{
    NotifyCocos(PokktCocosManager::VideoCompletedEvent.c_str(), "Video is completed");
}

- (void)onVideoDisplayed
{
    NotifyCocos(PokktCocosManager::VideoDisplayedEvent.c_str(), "Video is displayed");
}

- (void)onVideoClosed
{
    NotifyCocos(PokktCocosManager::VideoClosedEvent.c_str(), "Video is closed");
}

- (void)onVideoGratified:(VideoResponse *)videoResponse
{
    NotifyCocos(PokktCocosManager::VideoGratifiedEvent.c_str(), [[NSString stringWithFormat:@"%@", videoResponse.coins] UTF8String]);
}

- (void)onVideoSkiped
{
    NotifyCocos(PokktCocosManager::VideoSkippedEvent.c_str(), "Video is skipped");
}

void notifyIOS(const char* operation, const char* param)
{
    NSString *paramNSStr = [NSString stringWithUTF8String:(param)];
    
    // common methods
    
    if (std::strcmp(operation, "setParams") == 0)
    {
        NSArray *paramArray = [paramNSStr componentsSeparatedByString:@","];
        pokktConfig  = [[PokktConfig alloc]init];
        
        pokktConfig.applicationId = [paramArray objectAtIndex:1];
        pokktConfig.securityKey = [paramArray objectAtIndex:0];
        pokktConfig.autoCacheVideo = [[paramArray objectAtIndex:3] boolValue];
        
        [PokktManager initPokkt:pokktConfig];
        
        IOSExtension *ext = [[IOSExtension alloc] init];
        [PokktManager setPokktDelegate:ext];
    }
    
    else if (std::strcmp(operation, "setDebug") == 0)
    {
        BOOL debug = [paramNSStr boolValue];
        [PokktManager setDebug:debug];
    }
    
    else if (std::strcmp(operation, "showLog") == 0)
    {
        NSLog(@"[POKKTLOG] %@",paramNSStr);
    }
    
    else if (std::strcmp(operation, "showToast") == 0)
    {
        NSLog(@"[POKKTLOG] %@",@"iOS plugin does not support showToast!!");
    }
    
    else if (std::strcmp(operation, "setThirdPartyUserId") == 0)
    {
        [pokktConfig setThirdPartyUserId:paramNSStr];
    }
    
    // session methods
    
    else if(std::strcmp(operation, "startSession") == 0)
    {
        [PokktManager startSession:pokktConfig];
    }
    
    else if(std::strcmp(operation, "endSession") == 0)
    {
        [PokktManager endSession];
    }
    
    // OfferWall methods
    // all offerwall methods should log as error in default case
    
    // Video Methods
    
    else if (std::strcmp(operation, "getVideo") == 0)
    {
        // set the view controller
        UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
        [PokktManager setPresentView:viewController];
        pokktConfig.screenName = paramNSStr;
        pokktConfig.incentivised = YES;
        [PokktManager playVideoCampaign:pokktConfig];
    }
    
    else if (std::strcmp(operation, "getVideoNonIncent") == 0)
    {
        // set the view controller
        UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
        [PokktManager setPresentView:viewController];
        pokktConfig.screenName = paramNSStr;
        pokktConfig.incentivised = NO;
        [PokktManager playVideoCampaign:pokktConfig];
    }
    
    else if (std::strcmp(operation, "cacheVideoCampaign") == 0)
    {
        [PokktManager cacheVideoCampaign:pokktConfig];
    }
    
    else if (std::strcmp(operation, "setSkipEnabled") == 0)
    {
        [pokktConfig setSkipEnabled:(std::strcmp(param, "true") == 0)];
    }
    
    else if (std::strcmp(operation, "setDefaultSkipTime") == 0)
    {
        [pokktConfig setDefaultSkipTime:[paramNSStr intValue]];
    }
    
    else if (std::strcmp(operation, "setCustomSkipMessage") == 0)
    {
        [pokktConfig setCustomSkipMessage:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setBackButtonDisabled") == 0)
    {
        NSLog(@"[POKKTLOG] %@",@"iOS plugin does not support setBackButtonDisabled!!");
    }
    
    // optional methods
    
    else if (std::strcmp(operation, "setName") == 0)
    {
        [pokktConfig setName:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setAge") == 0)
    {
        [pokktConfig setAge:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setSex") == 0)
    {
        [pokktConfig setSex:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setMobileNo") == 0)
    {
        [pokktConfig setMobileNo:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setEmailAddress") == 0)
    {
        [pokktConfig setEmailAddress:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setLocation") == 0)
    {
        [pokktConfig setLocation:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setBirthday") == 0)
    {
        [pokktConfig setBirthday:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setMaritalStatus") == 0)
    {
        [pokktConfig setMaritalStatus:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setFacebookId") == 0)
    {
        [pokktConfig setFacebookId:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setTwitterHandle") == 0)
    {
        [pokktConfig setTwitterHandle:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setEducation") == 0)
    {
        [pokktConfig setEducation:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setNationality") == 0)
    {
        [pokktConfig setNationality:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setEmployment") == 0)
    {
        [pokktConfig setEmployment:paramNSStr];
    }
    
    else if (std::strcmp(operation, "setMaturityRating") == 0)
    {
        [pokktConfig setMaturityRating:paramNSStr];
    }
    
    else
    {
        NSLog(@"[POKKTLOG] %@",[NSString stringWithFormat:@"iOS plugin does not support %s operation",operation]);
    }
}

bool isVideoAvailableOnIOS()
{
    return [PokktManager isVideoAvailable];
}

double getVideoVcOnIOS()
{
    return [PokktManager getVideoVc];
}

const char* getPokktSDKVersionInIOS()
{
    const char* version = [[NSString stringWithFormat:@"%@", [PokktManager getSDKVersion]] UTF8String];
    return version;
}

@end
